@@ -14,6 +14,8 @@ extern "C" {
 
 #include <spa/utils/hook.h>
 
+#include <pipewire/type.h>
+
 /** \defgroup pw_core Core
  *
  * \brief The core global object.
@@ -41,6 +43,13 @@ struct pw_core;
 #define PW_VERSION_REGISTRY	3
 struct pw_registry;
 
+#ifndef PW_API_CORE_IMPL
+#define PW_API_CORE_IMPL static inline
+#endif
+#ifndef PW_API_REGISTRY_IMPL
+#define PW_API_REGISTRY_IMPL static inline
+#endif
+
 /** The default remote name to connect to */
 #define PW_DEFAULT_REMOTE	"pipewire-0"
 
@@ -334,23 +343,51 @@ struct pw_core_methods {
 	int (*destroy) (void *object, void *proxy);
 };
 
-#define pw_core_method(o,method,version,...)			\
-({									\
-	int _res = -ENOTSUP;						\
-	spa_interface_call_res((struct spa_interface*)o,		\
-			struct pw_core_methods, _res,		\
-			method, version, ##__VA_ARGS__);		\
-	_res;								\
-})
 
-#define pw_core_add_listener(c,...)	pw_core_method(c,add_listener,0,__VA_ARGS__)
-#define pw_core_hello(c,...)		pw_core_method(c,hello,0,__VA_ARGS__)
-#define pw_core_sync(c,...)		pw_core_method(c,sync,0,__VA_ARGS__)
-#define pw_core_pong(c,...)		pw_core_method(c,pong,0,__VA_ARGS__)
-#define pw_core_error(c,...)		pw_core_method(c,error,0,__VA_ARGS__)
-
-
-static inline
+/** \copydoc pw_core_methods.add_listener
+ * \sa pw_core_methods.add_listener */
+PW_API_CORE_IMPL int pw_core_add_listener(struct pw_core *object,
+			struct spa_hook *listener,
+			const struct pw_core_events *events,
+			void *data)
+{
+	return spa_api_method_r(int, -ENOTSUP,
+			pw_core, (struct spa_interface*)object, add_listener, 0,
+			listener, events, data);
+}
+/** \copydoc pw_core_methods.hello
+ * \sa pw_core_methods.hello */
+PW_API_CORE_IMPL int pw_core_hello(struct pw_core *object, uint32_t version)
+{
+	return spa_api_method_r(int, -ENOTSUP,
+			pw_core, (struct spa_interface*)object, hello, 0,
+			version);
+}
+/** \copydoc pw_core_methods.sync
+ * \sa pw_core_methods.sync */
+PW_API_CORE_IMPL int pw_core_sync(struct pw_core *object, uint32_t id, int seq)
+{
+	return spa_api_method_r(int, -ENOTSUP,
+			pw_core, (struct spa_interface*)object, sync, 0,
+			id, seq);
+}
+/** \copydoc pw_core_methods.pong
+ * \sa pw_core_methods.pong */
+PW_API_CORE_IMPL int pw_core_pong(struct pw_core *object, uint32_t id, int seq)
+{
+	return spa_api_method_r(int, -ENOTSUP,
+			pw_core, (struct spa_interface*)object, pong, 0,
+			id, seq);
+}
+/** \copydoc pw_core_methods.error
+ * \sa pw_core_methods.error */
+PW_API_CORE_IMPL int pw_core_error(struct pw_core *object, uint32_t id, int seq, int res, const char *message)
+{
+	return spa_api_method_r(int, -ENOTSUP,
+			pw_core, (struct spa_interface*)object, error, 0,
+			id, seq, res, message);
+}
+PW_API_CORE_IMPL
 SPA_PRINTF_FUNC(5, 0) int
 pw_core_errorv(struct pw_core *core, uint32_t id, int seq,
 		int res, const char *message, va_list args)
@@ -361,7 +398,7 @@ pw_core_errorv(struct pw_core *core, uint32_t id, int seq,
 	return pw_core_error(core, id, seq, res, buffer);
 }
 
-static inline
+PW_API_CORE_IMPL
 SPA_PRINTF_FUNC(5, 6) int
 pw_core_errorf(struct pw_core *core, uint32_t id, int seq,
 		int res, const char *message, ...)
@@ -374,17 +411,18 @@ pw_core_errorf(struct pw_core *core, uint32_t id, int seq,
 	return r;
 }
 
-static inline struct pw_registry *
+/** \copydoc pw_core_methods.get_registry
+ * \sa pw_core_methods.get_registry */
+PW_API_CORE_IMPL struct pw_registry *
 pw_core_get_registry(struct pw_core *core, uint32_t version, size_t user_data_size)
 {
-	struct pw_registry *res = NULL;
-	spa_interface_call_res((struct spa_interface*)core,
-			struct pw_core_methods, res,
-			get_registry, 0, version, user_data_size);
-	return res;
+	return spa_api_method_r(struct pw_registry*, NULL,
+			pw_core, (struct spa_interface*)core, get_registry, 0,
+			version, user_data_size);
 }
-
-static inline void *
+/** \copydoc pw_core_methods.create_object
+ * \sa pw_core_methods.create_object */
+PW_API_CORE_IMPL void *
 pw_core_create_object(struct pw_core *core,
 			    const char *factory_name,
 			    const char *type,
@@ -392,15 +430,18 @@ pw_core_create_object(struct pw_core *core,
 			    const struct spa_dict *props,
 			    size_t user_data_size)
 {
-	void *res = NULL;
-	spa_interface_call_res((struct spa_interface*)core,
-			struct pw_core_methods, res,
-			create_object, 0, factory_name,
-			type, version, props, user_data_size);
-	return res;
+	return spa_api_method_r(void*, NULL,
+			pw_core, (struct spa_interface*)core, create_object, 0,
+			factory_name, type, version, props, user_data_size);
+}
+/** \copydoc pw_core_methods.destroy
+ * \sa pw_core_methods.destroy */
+PW_API_CORE_IMPL void
+pw_core_destroy(struct pw_core *core, void *proxy)
+{
+	spa_api_method_v(pw_core, (struct spa_interface*)core, destroy, 0,
+			proxy);
 }
-
-#define pw_core_destroy(c,...)		pw_core_method(c,destroy,0,__VA_ARGS__)
 
 /**
  * \}
@@ -516,31 +557,38 @@ struct pw_registry_methods {
 	int (*destroy) (void *object, uint32_t id);
 };
 
-#define pw_registry_method(o,method,version,...)			\
-({									\
-	int _res = -ENOTSUP;						\
-	spa_interface_call_res((struct spa_interface*)o,		\
-			struct pw_registry_methods, _res,		\
-			method, version, ##__VA_ARGS__);		\
-	_res;								\
-})
 
 /** Registry */
-#define pw_registry_add_listener(p,...)	pw_registry_method(p,add_listener,0,__VA_ARGS__)
-
-static inline void *
+/** \copydoc pw_registry_methods.add_listener
+ * \sa pw_registry_methods.add_listener */
+PW_API_REGISTRY_IMPL int pw_registry_add_listener(struct pw_registry *registry,
+			struct spa_hook *listener,
+			const struct pw_registry_events *events,
+			void *data)
+{
+	return spa_api_method_r(int, -ENOTSUP,
+			pw_registry, (struct spa_interface*)registry, add_listener, 0,
+			listener, events, data);
+}
+/** \copydoc pw_registry_methods.bind
+ * \sa pw_registry_methods.bind */
+PW_API_REGISTRY_IMPL void *
 pw_registry_bind(struct pw_registry *registry,
 		       uint32_t id, const char *type, uint32_t version,
 		       size_t user_data_size)
 {
-	void *res = NULL;
-	spa_interface_call_res((struct spa_interface*)registry,
-			struct pw_registry_methods, res,
-			bind, 0, id, type, version, user_data_size);
-	return res;
+	return spa_api_method_r(void*, NULL,
+			pw_registry, (struct spa_interface*)registry, bind, 0,
+			id, type, version, user_data_size);
+}
+/** \copydoc pw_registry_methods.destroy
+ * \sa pw_registry_methods.destroy */
+PW_API_REGISTRY_IMPL int
+pw_registry_destroy(struct pw_registry *registry, uint32_t id)
+{
+	return spa_api_method_r(int, -ENOTSUP,
+			pw_registry, (struct spa_interface*)registry, destroy, 0, id);
 }
-
-#define pw_registry_destroy(p,...)	pw_registry_method(p,destroy,0,__VA_ARGS__)
 
 /**
  * \}
